-module(eensy@otp@actor).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([continue/1, with_selector/2, to_erlang_start_result/1, start_spec/1, start/2, send/2, call/3]).
-export_type([message/1, next/2, init_result/2, self/2, spec/2, start_error/0, start_init_message/1]).

-type message(GJO) :: {message, GJO} |
    {system, eensy@otp@system:system_message()} |
    {unexpected, gleam@dynamic:dynamic_()}.

-type next(GJP, GJQ) :: {continue,
        GJQ,
        gleam@option:option(gleam@erlang@process:selector(GJP))} |
    {stop, gleam@erlang@process:exit_reason()}.

-type init_result(GJR, GJS) :: {ready, GJR, gleam@erlang@process:selector(GJS)} |
    {failed, binary()}.

-type self(GJT, GJU) :: {self,
        eensy@otp@system:mode(),
        gleam@erlang@process:pid_(),
        GJT,
        gleam@erlang@process:subject(GJU),
        gleam@erlang@process:selector(message(GJU)),
        fun((GJU, GJT) -> next(GJU, GJT))}.

-type spec(GJV, GJW) :: {spec,
        fun(() -> init_result(GJV, GJW)),
        integer(),
        fun((GJW, GJV) -> next(GJW, GJV))}.

-type start_error() :: init_timeout |
    {init_failed, gleam@erlang@process:exit_reason()} |
    {init_crashed, gleam@dynamic:dynamic_()}.

-type start_init_message(GJX) :: {ack,
        {ok, gleam@erlang@process:subject(GJX)} |
            {error, gleam@erlang@process:exit_reason()}} |
    {mon, gleam@erlang@process:process_down()}.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 177).
-spec continue(GKE) -> next(any(), GKE).
continue(State) ->
    {continue, State, none}.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 185).
-spec with_selector(next(GKI, GKJ), gleam@erlang@process:selector(GKI)) -> next(GKI, GKJ).
with_selector(Value, Selector) ->
    case Value of
        {continue, State, _} ->
            {continue, State, {some, Selector}};

        _ ->
            Value
    end.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 258).
-spec exit_process(gleam@erlang@process:exit_reason()) -> gleam@erlang@process:exit_reason().
exit_process(Reason) ->
    Reason.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 263).
-spec receive_message(self(any(), GKQ)) -> message(GKQ).
receive_message(Self) ->
    Selector = case erlang:element(2, Self) of
        running ->
            _pipe = gleam_erlang_ffi:new_selector(),
            _pipe@1 = gleam@erlang@process:selecting_anything(
                _pipe,
                fun(Field@0) -> {unexpected, Field@0} end
            ),
            gleam_erlang_ffi:merge_selector(_pipe@1, erlang:element(6, Self));

        suspended ->
            _pipe = gleam_erlang_ffi:new_selector(),
            _pipe@1 = gleam@erlang@process:selecting_anything(
                _pipe,
                fun(Field@0) -> {unexpected, Field@0} end
            ),
            gleam_erlang_ffi:merge_selector(_pipe@1, erlang:element(6, Self))
    end,
    gleam_erlang_ffi:select(Selector).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 392).
-spec init_selector(
    gleam@erlang@process:subject(GOF),
    gleam@erlang@process:selector(GOF)
) -> gleam@erlang@process:selector(message(GOF)).
init_selector(Subject, Selector) ->
    _pipe = gleam_erlang_ffi:new_selector(),
    _pipe@1 = gleam@erlang@process:selecting(
        _pipe,
        Subject,
        fun(Field@0) -> {message, Field@0} end
    ),
    gleam_erlang_ffi:merge_selector(
        _pipe@1,
        gleam_erlang_ffi:map_selector(
            Selector,
            fun(Field@0) -> {message, Field@0} end
        )
    ).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 294).
-spec loop(self(any(), any())) -> gleam@erlang@process:exit_reason().
loop(Self) ->
    case receive_message(Self) of
        {system, System} ->
            case System of
                {get_state, Callback} ->
                    Callback(gleam_stdlib:identity(erlang:element(4, Self))),
                    loop(Self);

                {resume, Callback@1} ->
                    Callback@1(),
                    loop(erlang:setelement(2, Self, running));

                {suspend, Callback@2} ->
                    Callback@2(),
                    loop(erlang:setelement(2, Self, suspended));

                {get_status, _} ->
                    loop(Self)
            end;

        {unexpected, Message} ->
            logger:warning(
                unicode:characters_to_list(
                    <<"Actor discarding unexpected message: ~s"/utf8>>
                ),
                [unicode:characters_to_list(gleam@string:inspect(Message))]
            ),
            loop(Self);

        {message, Msg} ->
            case (erlang:element(7, Self))(Msg, erlang:element(4, Self)) of
                {stop, Reason} ->
                    exit_process(Reason);

                {continue, State, New_selector} ->
                    Selector = begin
                        _pipe = New_selector,
                        _pipe@1 = gleam@option:map(
                            _pipe,
                            fun(_capture) ->
                                init_selector(erlang:element(5, Self), _capture)
                            end
                        ),
                        gleam@option:unwrap(_pipe@1, erlang:element(6, Self))
                    end,
                    loop(
                        erlang:setelement(
                            6,
                            erlang:setelement(4, Self, State),
                            Selector
                        )
                    )
            end
    end.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 350).
-spec initialise_actor(
    spec(any(), GLA),
    gleam@erlang@process:subject({ok, gleam@erlang@process:subject(GLA)} |
        {error, gleam@erlang@process:exit_reason()})
) -> gleam@erlang@process:exit_reason().
initialise_actor(Spec, Ack) ->
    Subject = gleam@erlang@process:new_subject(),
    Result = (erlang:element(2, Spec))(),
    case Result of
        {ready, State, Selector} ->
            Selector@1 = init_selector(Subject, Selector),
            gleam@erlang@process:send(Ack, {ok, Subject}),
            Self = {self,
                running,
                gleam@erlang@process:subject_owner(Ack),
                State,
                Subject,
                Selector@1,
                erlang:element(4, Spec)},
            loop(Self);

        {failed, Reason} ->
            gleam@erlang@process:send(Ack, {error, {abnormal, Reason}}),
            exit_process({abnormal, Reason})
    end.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 421).
-spec to_erlang_start_result(
    {ok, gleam@erlang@process:subject(any())} | {error, start_error()}
) -> {ok, gleam@erlang@process:pid_()} | {error, gleam@dynamic:dynamic_()}.
to_erlang_start_result(Res) ->
    case Res of
        {ok, X} ->
            {ok, gleam@erlang@process:subject_owner(X)};

        {error, X@1} ->
            {error, gleam_stdlib:identity(X@1)}
    end.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 443).
-spec start_spec(spec(any(), GLN)) -> {ok, gleam@erlang@process:subject(GLN)} |
    {error, start_error()}.
start_spec(Spec) ->
    Ack_subject = gleam@erlang@process:new_subject(),
    Child = gleam@erlang@process:start(
        fun() -> initialise_actor(Spec, Ack_subject) end,
        true
    ),
    Monitor = gleam@erlang@process:monitor_process(Child),
    Selector = begin
        _pipe = gleam_erlang_ffi:new_selector(),
        _pipe@1 = gleam@erlang@process:selecting(
            _pipe,
            Ack_subject,
            fun(Field@0) -> {ack, Field@0} end
        ),
        gleam@erlang@process:selecting_process_down(
            _pipe@1,
            Monitor,
            fun(Field@0) -> {mon, Field@0} end
        )
    end,
    Result = case gleam_erlang_ffi:select(Selector, erlang:element(3, Spec)) of
        {ok, {ack, {ok, Channel}}} ->
            {ok, Channel};

        {ok, {ack, {error, Reason}}} ->
            {error, {init_failed, Reason}};

        {ok, {mon, Down}} ->
            {error, {init_crashed, erlang:element(3, Down)}};

        {error, nil} ->
            gleam@erlang@process:kill(Child),
            {error, init_timeout}
    end,
    Result.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 492).
-spec start(GLT, fun((GLU, GLT) -> next(GLU, GLT))) -> {ok,
        gleam@erlang@process:subject(GLU)} |
    {error, start_error()}.
start(State, Loop) ->
    start_spec(
        {spec,
            fun() -> {ready, State, gleam_erlang_ffi:new_selector()} end,
            5000,
            Loop}
    ).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 507).
-spec send(gleam@erlang@process:subject(GMA), GMA) -> nil.
send(Subject, Msg) ->
    gleam@erlang@process:send(Subject, Msg).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/actor.gleam", 520).
-spec call(
    gleam@erlang@process:subject(GMC),
    fun((gleam@erlang@process:subject(GME)) -> GMC),
    integer()
) -> GME.
call(Subject, Make_message, Timeout) ->
    gleam@erlang@process:call(Subject, Make_message, Timeout).
