-module(eensy@otp@static_supervisor).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([new/1, restart_tolerance/3, auto_shutdown/2, add/2, worker_child/2, supervisor_child/2, significant/2, timeout/2, restart/2, start_link/1, init/1]).
-export_type([strategy/0, auto_shutdown/0, builder/0, restart/0, child_type/0, child_builder/0]).

-type strategy() :: one_for_one | one_for_all | rest_for_one.

-type auto_shutdown() :: never | any_significant | all_significant.

-opaque builder() :: {builder,
        strategy(),
        integer(),
        integer(),
        auto_shutdown(),
        list(child_builder())}.

-type restart() :: permanent | transient | temporary.

-type child_type() :: {worker, integer()} | supervisor.

-opaque child_builder() :: {child_builder,
        binary(),
        fun(() -> {ok, gleam@erlang@process:pid_()} |
            {error, gleam@dynamic:dynamic_()}),
        restart(),
        boolean(),
        child_type()}.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 73).
-spec new(strategy()) -> builder().
new(Strategy) ->
    {builder, Strategy, 2, 5, never, []}.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 93).
-spec restart_tolerance(builder(), integer(), integer()) -> builder().
restart_tolerance(Builder, Intensity, Period) ->
    erlang:setelement(4, erlang:setelement(3, Builder, Intensity), Period).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 103).
-spec auto_shutdown(builder(), auto_shutdown()) -> builder().
auto_shutdown(Builder, Value) ->
    erlang:setelement(5, Builder, Value).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 191).
-spec add(builder(), child_builder()) -> builder().
add(Builder, Child) ->
    erlang:setelement(6, Builder, [Child | erlang:element(6, Builder)]).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 204).
-spec worker_child(
    binary(),
    fun(() -> {ok, gleam@erlang@process:pid_()} | {error, any()})
) -> child_builder().
worker_child(Id, Starter) ->
    {child_builder, Id, fun() -> _pipe = Starter(),
            gleam@result:map_error(_pipe, fun gleam_stdlib:identity/1) end, permanent, false, {worker,
            5000}}.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 226).
-spec supervisor_child(
    binary(),
    fun(() -> {ok, gleam@erlang@process:pid_()} | {error, any()})
) -> child_builder().
supervisor_child(Id, Starter) ->
    {child_builder, Id, fun() -> _pipe = Starter(),
            gleam@result:map_error(_pipe, fun gleam_stdlib:identity/1) end, permanent, false, supervisor}.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 248).
-spec significant(child_builder(), boolean()) -> child_builder().
significant(Child, Significant) ->
    erlang:setelement(5, Child, Significant).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 259).
-spec timeout(child_builder(), integer()) -> child_builder().
timeout(Child, Ms) ->
    case erlang:element(6, Child) of
        {worker, _} ->
            erlang:setelement(6, Child, {worker, Ms});

        _ ->
            Child
    end.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 270).
-spec restart(child_builder(), restart()) -> child_builder().
restart(Child, Restart) ->
    erlang:setelement(4, Child, Restart).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 274).
-spec convert_child(child_builder()) -> {binary(),
    {gleam@erlang@atom:atom_(),
        gleam@erlang@atom:atom_(),
        list(gleam@dynamic:dynamic_())},
    restart(),
    gleam@erlang@atom:atom_(),
    gleam@erlang@atom:atom_(),
    list(gleam@dynamic:dynamic_())}.
convert_child(Child) ->
    Mfa = {erlang:binary_to_atom(<<"erlang"/utf8>>),
        erlang:binary_to_atom(<<"apply"/utf8>>),
        [gleam_stdlib:identity(erlang:element(3, Child)),
            gleam_stdlib:identity([])]},
    {Type_, Shutdown} = case erlang:element(6, Child) of
        supervisor ->
            {erlang:binary_to_atom(<<"supervisor"/utf8>>),
                erlang:binary_to_atom(<<"brutal_kill"/utf8>>)};

        {worker, _} ->
            {erlang:binary_to_atom(<<"worker"/utf8>>),
                erlang:binary_to_atom(<<"brutal_kill"/utf8>>)}
    end,
    {erlang:element(2, Child),
        Mfa,
        erlang:element(4, Child),
        Shutdown,
        Type_,
        [gleam_stdlib:identity(<<"erlang"/utf8>>)]}.

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 164).
-spec start_link(builder()) -> {ok, gleam@erlang@process:pid_()} |
    {error, gleam@dynamic:dynamic_()}.
start_link(Builder) ->
    Children = begin
        _pipe = erlang:element(6, Builder),
        _pipe@1 = lists:reverse(_pipe),
        gleam@list:map(_pipe@1, fun convert_child/1)
    end,
    eensy_otp_ffi:static_supervisor_start_link(
        {{erlang:element(2, Builder),
                erlang:element(3, Builder),
                erlang:element(4, Builder)},
            Children}
    ).

-file("/Users/raynerdls/ray.delossantos/apps/eensy-repos/packages/eensy/src/eensy/otp/static_supervisor.gleam", 299).
-spec init(gleam@dynamic:dynamic_()) -> {ok, gleam@dynamic:dynamic_()} |
    {error, any()}.
init(Start_data) ->
    {ok, Start_data}.
